
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <termios.h>

// Variables
char *device0 = "/dev/ttyS0";
char *device1 = "/dev/ttyS1";
char *device2 = "/dev/ttyS2";
char *device3 = "/dev/ttyS3";
int fd;
struct termios tty;
char c;
float dose;
float coeff;
float cps;
float avg;
float avgh;
float level;
int nacq;
int noct;
int k;
unsigned char cks;
unsigned char header;
unsigned char msb;
unsigned char lsb;
unsigned char restart;
unsigned int timout;
char reply[11];

// Prototypes
void receive(void);
void record(void);
void opcom(void);

int main(int argc, char *argv[])
	{
	restart=1;
	coeff=0;
	printf("\nRadioactivity exposure rate - www.vptec.narod.ru\n");
	printf("To be used with serial data connection GM counter described on the web site\n\n");
	printf("0 - Select ttyS0\n1 - Select ttyS1\n2 - Select ttyS2\n3 - Select ttyS3\nQ - Abort\n\n");
	printf("Enter selection: ");
	scanf("%10s",reply);
	switch (reply[0])
		{
		case '0':
		break;
		case '1':
		strcpy(device0,device1);
		break;
		case '2':
		strcpy(device0,device2);
		break;
		case '3':
		strcpy(device0,device3);
		break;
		default:
		restart=0;
		break;
		}
	while (coeff<=0)
		{
		printf("Enter GM tube coefficient in CPM/Roentgen/h: ");
		scanf("%10s",reply);
		sscanf(reply,"%f",&coeff);
		}
	while (restart==1)
		{
		dose=0;
		avg=0;
		avgh=0;
		nacq=0;
		noct=0;
		cks=0;
		header=0;
		msb=0;
		lsb=0;
		cps=0;
		c=0;
		timout=0;
		printf("\nPress 'G' + enter when ready to start measurement or 'Q' + enter to quit... ");
		scanf("%2s",reply);
		if ((reply[0]!='g')&&(reply[0]!='G'))
			{
			exit(0);
			}
		opcom();
		if(!fd)
			{
			exit(0);
			}
		printf("\n\nWaiting for data (12s time-out)...\n");
		while (timout<12000)
			{
			timout+=1;
			usleep(1000);
			receive();
			}
		close(fd);
		printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nGM counter stopped !\n\n");
		printf("Start new measurements (y/n) ? ");
		scanf("%2s",reply);
		printf("\n\n");
		if((reply[0]!='y')&&(reply[0]!='Y'))
			{
			restart=0;
			}
		}
	}

void receive(void)
	{
	if(read(fd,&c,1))
		{
		switch (noct)
			{
			case 0:
			header=(unsigned char)c;
			break;
			case 1:
			lsb=(unsigned char)c;
			break;
			case 2:
			msb=(unsigned char)c;
			break;
			case 3:
			cks=header+msb+lsb;
			if((header==(unsigned char)0xFF)&&(cks==(unsigned char)c))
				{
				record();
				timout=0;
				noct=0;
				}
			else
				{
				header=lsb;
				lsb=msb;
				msb=(unsigned char)c;
				noct-=1;
				}
			break;
			}
		noct+=1;
		timout=0;
		}
	}

void record(void)
	{
	nacq+=1;
	if(nacq>1)
		{
		cps+=(float)((((unsigned int)msb)<<8)+(unsigned int)lsb);
		printf("%c[%cACounts over %d seconds: %f counts        \n",0x1B,1,(10*(nacq-1)),cps);
		if (nacq==7)
			{
			nacq=1;
			level = cps / coeff;
			cps=0;
			// Print converted values
			printf("\n\nOver the last minute (gamma assumption - QF=1):\n\n");
			printf("* %f mR/h          \n",1000*level);
			level/=8.38;
			printf("* %f rad/h          \n",100*level);
			printf("* %f rem/h          \n",100*level);
			printf("* %f uSv/h          \n",1000000*level);
			printf("* %f uGy/h         \n\n",1000000*level);
			printf("Dose since measurement started:\n\n");
			dose+=(level*146000);
			printf("* %f mSv          \n",dose);
			printf("* %f rem          \n",dose/10);
			for (k=0;k!=14;k+=1)
				{
				printf("%c[%cA",0x1B,1);
				}
			}
		}
	else
		{
		printf("%c[%cAReception started !                      \n",0x1B,1);
		}
	}

void opcom(void)
	{
	printf("Opening %s ...\n\n",device0);
	fd = open(device0,O_RDWR | O_NOCTTY | O_NONBLOCK);
	if(!fd)
		{
		printf("Com Port Opening Failed !\n");
		return;
		}
	tty.c_cflag = B1200;			/* 1200 bauds */
	tty.c_cflag |= CLOCAL;			/* ignore modem control lines */
	tty.c_cflag |= CREAD;			/* enable receiver */
	tty.c_cflag |= CS8;			/* use 8 data bits */
	tty.c_cflag |= CSTOPB;			/* 2 stop bits */
	tty.c_cflag &= ~PARENB;			/* disable parity */
	//tty.c_cflag |= PARODD;		/* use odd parity */
	tty.c_cflag &= ~CRTSCTS;		/* do not use RTS and CTS handshake */
	tcsetattr(fd,TCSANOW,&tty);
	printf("Port opened successfully (Stop GM counter when need to interrupt)\n");
	}



